Imports System
Imports Microsoft.Win32.SafeHandles
Imports System.Runtime.InteropServices

Namespace WinUsbDemo
	''' <summary>
	'''  These declarations are translated from the C declarations in various files
	'''  in the WDK:
	'''  
	'''  usb.h
	'''  usb100.h
	'''  winusbio.h	
	''' </summary>

	Friend NotInheritable Partial Class WinUsbCommunications
		Friend NotInheritable Class NativeMethods

			Private Sub New()
			End Sub

			Friend Const DEVICE_SPEED As UInt32 = 1
			Friend Const USB_ENDPOINT_DIRECTION_MASK As Byte = &H80

			Friend Enum POLICY_TYPE
				SHORT_PACKET_TERMINATE = 1
				AUTO_CLEAR_STALL
				PIPE_TRANSFER_TIMEOUT
				IGNORE_SHORT_PACKETS
				ALLOW_PARTIAL_READS
				AUTO_FLUSH
				RAW_IO
			End Enum

			Friend Enum USBD_PIPE_TYPE
				UsbdPipeTypeControl
				UsbdPipeTypeIsochronous
				UsbdPipeTypeBulk
				UsbdPipeTypeInterrupt
			End Enum

			Friend Enum USB_DEVICE_SPEED
				UsbLowSpeed = 1
				UsbFullSpeed
				UsbHighSpeed
			End Enum

			<StructLayout(LayoutKind.Sequential)> _
			Friend Structure USB_CONFIGURATION_DESCRIPTOR
				Friend bLength As Byte
				Friend bDescriptorType As Byte
				Friend wTotalLength As UShort
				Friend bNumInterfaces As Byte
				Friend bConfigurationValue As Byte
				Friend iConfiguration As Byte
				Friend bmAttributes As Byte
				Friend MaxPower As Byte
			End Structure

			<StructLayout(LayoutKind.Sequential)> _
			Friend Structure USB_INTERFACE_DESCRIPTOR
				Friend bLength As Byte
				Friend bDescriptorType As Byte
				Friend bInterfaceNumber As Byte
				Friend bAlternateSetting As Byte
				Friend bNumEndpoints As Byte
				Friend bInterfaceClass As Byte
				Friend bInterfaceSubClass As Byte
				Friend bInterfaceProtocol As Byte
				Friend iInterface As Byte
			End Structure

			<StructLayout(LayoutKind.Sequential)> _
			Friend Structure WINUSB_PIPE_INFORMATION
				Friend PipeType As USBD_PIPE_TYPE
				Friend PipeId As Byte
				Friend MaximumPacketSize As UShort
				Friend Interval As Byte
			End Structure

			<StructLayout(LayoutKind.Sequential, Pack := 1)> _
			Friend Structure WINUSB_SETUP_PACKET
				Friend RequestType As Byte
				Friend Request As Byte
				Friend Value As UShort
				Friend Index As UShort
				Friend Length As UShort
			End Structure

			<DllImport("winusb.dll", SetLastError := True)> _
			Friend Shared Function WinUsb_ControlTransfer(ByVal InterfaceHandle As IntPtr, ByVal SetupPacket As WINUSB_SETUP_PACKET, ByVal Buffer() As Byte, ByVal BufferLength As UInt32, ByRef LengthTransferred As UInt32, ByVal Overlapped As IntPtr) As Boolean
			End Function

			<DllImport("winusb.dll", SetLastError := True)> _
			Friend Shared Function WinUsb_Free(ByVal InterfaceHandle As IntPtr) As Boolean
			End Function

			<DllImport("winusb.dll", SetLastError := True)> _
			Friend Shared Function WinUsb_Initialize(ByVal DeviceHandle As SafeFileHandle, ByRef InterfaceHandle As IntPtr) As Boolean
			End Function

			'  Use this declaration to retrieve DEVICE_SPEED (the only currently defined InformationType).

			<DllImport("winusb.dll", SetLastError := True)> _
			Friend Shared Function WinUsb_QueryDeviceInformation(ByVal InterfaceHandle As IntPtr, ByVal InformationType As UInt32, ByRef BufferLength As UInt32, ByRef Buffer As Byte) As Boolean
			End Function

			<DllImport("winusb.dll", SetLastError := True)> _
			Friend Shared Function WinUsb_QueryInterfaceSettings(ByVal InterfaceHandle As IntPtr, ByVal AlternateInterfaceNumber As Byte, ByRef UsbAltInterfaceDescriptor As USB_INTERFACE_DESCRIPTOR) As Boolean
			End Function

			<DllImport("winusb.dll", SetLastError := True)> _
			Friend Shared Function WinUsb_QueryPipe(ByVal InterfaceHandle As IntPtr, ByVal AlternateInterfaceNumber As Byte, ByVal PipeIndex As Byte, ByRef PipeInformation As WINUSB_PIPE_INFORMATION) As Boolean
			End Function

			<DllImport("winusb.dll", SetLastError := True)> _
			Friend Shared Function WinUsb_ReadPipe(ByVal InterfaceHandle As IntPtr, ByVal PipeID As Byte, ByVal Buffer() As Byte, ByVal BufferLength As UInt32, ByRef LengthTransferred As UInt32, ByVal Overlapped As IntPtr) As Boolean
			End Function

			'  Two declarations for WinUsb_SetPipePolicy. 
			'  Use this one when the returned Value is a Byte (all except PIPE_TRANSFER_TIMEOUT):

			<DllImport("winusb.dll", SetLastError := True)> _
			Friend Shared Function WinUsb_SetPipePolicy(ByVal InterfaceHandle As IntPtr, ByVal PipeID As Byte, ByVal PolicyType As UInt32, ByVal ValueLength As UInt32, ByRef Value As Byte) As Boolean
			End Function

			'  Use this alias when the returned Value is a UInt32 (PIPE_TRANSFER_TIMEOUT only):

			<DllImport("winusb.dll", SetLastError := True, EntryPoint := "WinUsb_SetPipePolicy")> _
			Friend Shared Function WinUsb_SetPipePolicy1(ByVal InterfaceHandle As IntPtr, ByVal PipeID As Byte, ByVal PolicyType As UInt32, ByVal ValueLength As UInt32, ByRef Value As UInt32) As Boolean
			End Function

			<DllImport("winusb.dll", SetLastError := True)> _
			Friend Shared Function WinUsb_WritePipe(ByVal InterfaceHandle As IntPtr, ByVal PipeID As Byte, ByVal Buffer() As Byte, ByVal BufferLength As UInt32, ByRef LengthTransferred As UInt32, ByVal Overlapped As IntPtr) As Boolean
			End Function
		End Class
	End Class
End Namespace
