Imports System
Imports System.Diagnostics
Imports System.Windows.Forms
Imports Microsoft.Win32.SafeHandles
Imports System.Management
Imports System.Text

Namespace WinUsbDemo


	''' <summary>
	''' Project: WinUsb_vb
	''' 
	''' ***********************************************************************
	''' Software License Agreement
	'''
	''' Licensor grants any person obtaining a copy of this software ("You") 
	''' a worldwide, royalty-free, non-exclusive license, for the duration of 
	''' the copyright, free of charge, to store and execute the Software in a 
	''' computer system and to incorporate the Software or any portion of it 
	''' in computer programs You write.   
	''' 
	''' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	''' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	''' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	''' AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	''' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	''' OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
	''' THE SOFTWARE.
	''' ***********************************************************************
	''' 
	''' Author
	''' Jan Axelson          
	''' 
	''' This software was created using Visual Studio 2012 Professional Edition with .NET Framework 4.0.
	''' I wrote the application in Visual C# and used Instant VB (http://www.tangiblesoftwaresolutions.com) to convert to Visual Basic.
	''' 
	''' Purpose: 
	''' Demonstrates USB communications using Microsoft's WinUSB driver.
	''' 
	''' Requirements:
	''' Windows XP or later and an attached USB device that uses the WinUSB driver.
	''' 
	''' Edit VendorID and ProductID to match the values in your device's device descriptor.
	''' If using the Windows-provided winusb.inf, edit DeviceInterfaceGuid to match the value in your device's extended properties OS feature descriptor .
	''' If using a vendor-provided INF file, edit DeviceInterfaceGuid to match the value in your INF file.
	''' 
	''' Description:
	''' Finds an attached device whose device firmware or host INF file contains a specific device interface GUID.
	''' Enables sending and receiving data via bulk, interrupt, and control transfers.
	''' 
	''' Uses WMI to detect when a device is attached or removed.
	''' 
	''' For bulk transfers, the application uses a Delegate and the BeginInvoke 
	''' and EndInvoke methods to read and write data asynchronously so the application's main thread 
	''' doesn't have to wait for the device to return data. A callback routine uses 
	''' marshaling to send data to the form, whose code runs in a different thread.
	''' 
	''' Interrupt and control transfers read and write data synchronously, blocking the thread
	''' until the operation completes or a timeout. (Normally, the wait for completion is minimal.)
	'''  
	''' This software and companion device firmware are available from http://www.Lvr.com/winusb.htm
	''' 
	''' Report any problems to jan@Lvr.com or post to my Ports forum at http://www.Lvr.com/forum 
	''' This application has been tested under Windows 8 (64 bit).
	''' 
	''' 10/15/13
	''' V2.1
	''' For detecting device arrival and removal, replaced the Win32 RegisterDeviceNotification code with .NET WMI functions.
	''' Added try...catch blocks.
	''' Other minor edits for clarity.
	''' 
	''' 5/3/13
	'''	V2.0
	'''	To illustrate asynchronous operations, bulk read and write are asynchronous.
	'''	
	'''	To illustrate synchronous operations, interrupt read and write and control transfers are synchronous. 
	'''	
	''' For compatibility, replaced ToInt32 with ToInt64 here:
	''' IntPtr pDevicePathName = new IntPtr(detailDataBuffer.ToInt64() + 4);
	''' 
	''' Uncommented QueryDeviceSpeed because Microsoft corrected the documentation for it.
	''' 
	''' Renamed some routines and variables.
	''' 
	''' Moved myDevInfo and other variables from WinUsbDevice.vb to frmMain.vb. 
	''' 
	''' Moved API declarations to classes named NativeMethods as recommended by Microsoft.
	''' 
	''' Minor changes to API declarations as advised by Visual Studio's Code Analysis.
	''' 
	''' Naming changes and other minor coding improvements as recommended by the Resharper tool.
	''' </summary>



	''' <summary>
	''' The application's form. Buttons enable detecting a device
	''' and initiating transfers.
	''' </summary>
	''' 
	Friend Class FrmMain
		Inherits Form
#Region "Windows Form Designer generated code "
		'#region '"Windows Form Designer generated code "'
		Public Sub New()
			MyBase.New()


			' This call is required by the Windows Form Designer.
			InitializeComponent()
		End Sub
		' Form overrides dispose to clean up the component list.
		Protected Overrides Sub Dispose(ByVal Disposing As Boolean)
			If Disposing Then
				If components IsNot Nothing Then
					components.Dispose()
				End If
			End If
			MyBase.Dispose(Disposing)
		End Sub

		' Required by the Windows Form Designer
		Private components As System.ComponentModel.IContainer
		Public ToolTip1 As System.Windows.Forms.ToolTip
		Public WithEvents CmdSendandReceiveViaBulkTransfers As System.Windows.Forms.Button
		Public FraControlReadTransfer As System.Windows.Forms.GroupBox
		Public TxtBulkDataToSend As System.Windows.Forms.TextBox
		Public FraBulkTransfer As System.Windows.Forms.GroupBox
		Public CboInterruptOutByte1 As System.Windows.Forms.ComboBox
		Public CboInterruptOutByte0 As System.Windows.Forms.ComboBox
		Public FraInterruptTransfer As System.Windows.Forms.GroupBox
		Public LstResults As System.Windows.Forms.ListBox
		' NOTE: The following procedure is required by the Windows Form Designer
		' It can be modified using the Windows Form Designer.
		' Do not modify it using the code editor.
		Friend WithEvents CmdFindDevice As System.Windows.Forms.Button
		Friend WithEvents CmdSendAndReceiveViaInterruptTransfers As System.Windows.Forms.Button
		Public FraControWriteTransfer As System.Windows.Forms.GroupBox
		Friend WithEvents CmdControlWriteTransfer As System.Windows.Forms.Button
		Public CboControlWriteByte1 As System.Windows.Forms.ComboBox
		Public CboControlWriteByte0 As System.Windows.Forms.ComboBox
		Friend WithEvents CmdControlReadTransfer As System.Windows.Forms.Button

		<System.Diagnostics.DebuggerStepThrough()> _
		Private Sub InitializeComponent()
			Me.components = New System.ComponentModel.Container()
			Me.ToolTip1 = New System.Windows.Forms.ToolTip(Me.components)
			Me.FraControlReadTransfer = New System.Windows.Forms.GroupBox()
			Me.CmdControlReadTransfer = New System.Windows.Forms.Button()
			Me.CmdSendandReceiveViaBulkTransfers = New System.Windows.Forms.Button()
			Me.FraBulkTransfer = New System.Windows.Forms.GroupBox()
			Me.TxtBulkDataToSend = New System.Windows.Forms.TextBox()
			Me.FraInterruptTransfer = New System.Windows.Forms.GroupBox()
			Me.CmdSendAndReceiveViaInterruptTransfers = New System.Windows.Forms.Button()
			Me.CboInterruptOutByte1 = New System.Windows.Forms.ComboBox()
			Me.CboInterruptOutByte0 = New System.Windows.Forms.ComboBox()
			Me.LstResults = New System.Windows.Forms.ListBox()
			Me.CmdFindDevice = New System.Windows.Forms.Button()
			Me.FraControWriteTransfer = New System.Windows.Forms.GroupBox()
			Me.CboControlWriteByte1 = New System.Windows.Forms.ComboBox()
			Me.CboControlWriteByte0 = New System.Windows.Forms.ComboBox()
			Me.CmdControlWriteTransfer = New System.Windows.Forms.Button()
			Me.FraControlReadTransfer.SuspendLayout()
			Me.FraBulkTransfer.SuspendLayout()
			Me.FraInterruptTransfer.SuspendLayout()
			Me.FraControWriteTransfer.SuspendLayout()
			Me.SuspendLayout()
			' 
			' FraControlReadTransfer
			' 
			Me.FraControlReadTransfer.BackColor = System.Drawing.SystemColors.Control
			Me.FraControlReadTransfer.Controls.Add(Me.CmdControlReadTransfer)
			Me.FraControlReadTransfer.Font = New System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.FraControlReadTransfer.ForeColor = System.Drawing.SystemColors.ControlText
			Me.FraControlReadTransfer.Location = New System.Drawing.Point(237, 27)
			Me.FraControlReadTransfer.Name = "FraControlReadTransfer"
			Me.FraControlReadTransfer.RightToLeft = System.Windows.Forms.RightToLeft.No
			Me.FraControlReadTransfer.Size = New System.Drawing.Size(175, 94)
			Me.FraControlReadTransfer.TabIndex = 7
			Me.FraControlReadTransfer.TabStop = False
			Me.FraControlReadTransfer.Text = "Control Read Transfer"
			' 
			' CmdControlReadTransfer
			' 
			Me.CmdControlReadTransfer.Font = New System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.CmdControlReadTransfer.Location = New System.Drawing.Point(49, 28)
			Me.CmdControlReadTransfer.Name = "CmdControlReadTransfer"
			Me.CmdControlReadTransfer.Size = New System.Drawing.Size(71, 36)
			Me.CmdControlReadTransfer.TabIndex = 12
			Me.CmdControlReadTransfer.Text = "Send"
			Me.CmdControlReadTransfer.UseVisualStyleBackColor = True
			'			Me.CmdControlReadTransfer.Click += New System.EventHandler(Me.cmdControlReadTransfer_Click)
			' 
			' CmdSendandReceiveViaBulkTransfers
			' 
			Me.CmdSendandReceiveViaBulkTransfers.BackColor = System.Drawing.SystemColors.Control
			Me.CmdSendandReceiveViaBulkTransfers.Cursor = System.Windows.Forms.Cursors.Default
			Me.CmdSendandReceiveViaBulkTransfers.Font = New System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.CmdSendandReceiveViaBulkTransfers.ForeColor = System.Drawing.SystemColors.ControlText
			Me.CmdSendandReceiveViaBulkTransfers.Location = New System.Drawing.Point(256, 24)
			Me.CmdSendandReceiveViaBulkTransfers.Name = "CmdSendandReceiveViaBulkTransfers"
			Me.CmdSendandReceiveViaBulkTransfers.RightToLeft = System.Windows.Forms.RightToLeft.No
			Me.CmdSendandReceiveViaBulkTransfers.Size = New System.Drawing.Size(71, 36)
			Me.CmdSendandReceiveViaBulkTransfers.TabIndex = 8
			Me.CmdSendandReceiveViaBulkTransfers.Text = "Send"
			Me.CmdSendandReceiveViaBulkTransfers.UseVisualStyleBackColor = False
			'			Me.CmdSendandReceiveViaBulkTransfers.Click += New System.EventHandler(Me.cmdSendAndReceiveViaBulkTransfers_Click)
			' 
			' FraBulkTransfer
			' 
			Me.FraBulkTransfer.BackColor = System.Drawing.SystemColors.Control
			Me.FraBulkTransfer.Controls.Add(Me.TxtBulkDataToSend)
			Me.FraBulkTransfer.Controls.Add(Me.CmdSendandReceiveViaBulkTransfers)
			Me.FraBulkTransfer.Font = New System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.FraBulkTransfer.ForeColor = System.Drawing.SystemColors.ControlText
			Me.FraBulkTransfer.Location = New System.Drawing.Point(16, 139)
			Me.FraBulkTransfer.Name = "FraBulkTransfer"
			Me.FraBulkTransfer.RightToLeft = System.Windows.Forms.RightToLeft.No
			Me.FraBulkTransfer.Size = New System.Drawing.Size(341, 89)
			Me.FraBulkTransfer.TabIndex = 4
			Me.FraBulkTransfer.TabStop = False
			Me.FraBulkTransfer.Text = "Bulk Transfers"
			' 
			' TxtBulkDataToSend
			' 
			Me.TxtBulkDataToSend.AcceptsReturn = True
			Me.TxtBulkDataToSend.BackColor = System.Drawing.SystemColors.Window
			Me.TxtBulkDataToSend.Cursor = System.Windows.Forms.Cursors.IBeam
			Me.TxtBulkDataToSend.Font = New System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.TxtBulkDataToSend.ForeColor = System.Drawing.SystemColors.WindowText
			Me.TxtBulkDataToSend.Location = New System.Drawing.Point(16, 19)
			Me.TxtBulkDataToSend.MaxLength = 0
			Me.TxtBulkDataToSend.Multiline = True
			Me.TxtBulkDataToSend.Name = "TxtBulkDataToSend"
			Me.TxtBulkDataToSend.RightToLeft = System.Windows.Forms.RightToLeft.No
			Me.TxtBulkDataToSend.ScrollBars = System.Windows.Forms.ScrollBars.Both
			Me.TxtBulkDataToSend.Size = New System.Drawing.Size(224, 59)
			Me.TxtBulkDataToSend.TabIndex = 5
			Me.TxtBulkDataToSend.Text = "bulk transfer test data"
			' 
			' FraInterruptTransfer
			' 
			Me.FraInterruptTransfer.BackColor = System.Drawing.SystemColors.Control
			Me.FraInterruptTransfer.Controls.Add(Me.CmdSendAndReceiveViaInterruptTransfers)
			Me.FraInterruptTransfer.Controls.Add(Me.CboInterruptOutByte1)
			Me.FraInterruptTransfer.Controls.Add(Me.CboInterruptOutByte0)
			Me.FraInterruptTransfer.Font = New System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.FraInterruptTransfer.ForeColor = System.Drawing.SystemColors.ControlText
			Me.FraInterruptTransfer.Location = New System.Drawing.Point(379, 139)
			Me.FraInterruptTransfer.Name = "FraInterruptTransfer"
			Me.FraInterruptTransfer.RightToLeft = System.Windows.Forms.RightToLeft.No
			Me.FraInterruptTransfer.Size = New System.Drawing.Size(180, 89)
			Me.FraInterruptTransfer.TabIndex = 1
			Me.FraInterruptTransfer.TabStop = False
			Me.FraInterruptTransfer.Text = "Interrupt Transfers"
			' 
			' CmdSendAndReceiveViaInterruptTransfers
			' 
			Me.CmdSendAndReceiveViaInterruptTransfers.Font = New System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.CmdSendAndReceiveViaInterruptTransfers.Location = New System.Drawing.Point(98, 24)
			Me.CmdSendAndReceiveViaInterruptTransfers.Name = "CmdSendAndReceiveViaInterruptTransfers"
			Me.CmdSendAndReceiveViaInterruptTransfers.Size = New System.Drawing.Size(71, 36)
			Me.CmdSendAndReceiveViaInterruptTransfers.TabIndex = 7
			Me.CmdSendAndReceiveViaInterruptTransfers.Text = "Send"
			Me.CmdSendAndReceiveViaInterruptTransfers.UseVisualStyleBackColor = True
			'			Me.CmdSendAndReceiveViaInterruptTransfers.Click += New System.EventHandler(Me.cmdSendAndReceiveViaInterruptTransfers_Click)
			' 
			' CboInterruptOutByte1
			' 
			Me.CboInterruptOutByte1.BackColor = System.Drawing.SystemColors.Window
			Me.CboInterruptOutByte1.Cursor = System.Windows.Forms.Cursors.Default
			Me.CboInterruptOutByte1.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList
			Me.CboInterruptOutByte1.Font = New System.Drawing.Font("Arial", 8.0F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.CboInterruptOutByte1.ForeColor = System.Drawing.SystemColors.WindowText
			Me.CboInterruptOutByte1.Location = New System.Drawing.Point(8, 56)
			Me.CboInterruptOutByte1.Name = "CboInterruptOutByte1"
			Me.CboInterruptOutByte1.RightToLeft = System.Windows.Forms.RightToLeft.No
			Me.CboInterruptOutByte1.Size = New System.Drawing.Size(74, 22)
			Me.CboInterruptOutByte1.TabIndex = 3
			' 
			' CboInterruptOutByte0
			' 
			Me.CboInterruptOutByte0.BackColor = System.Drawing.SystemColors.Window
			Me.CboInterruptOutByte0.Cursor = System.Windows.Forms.Cursors.Default
			Me.CboInterruptOutByte0.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList
			Me.CboInterruptOutByte0.Font = New System.Drawing.Font("Arial", 8.0F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.CboInterruptOutByte0.ForeColor = System.Drawing.SystemColors.WindowText
			Me.CboInterruptOutByte0.Location = New System.Drawing.Point(8, 24)
			Me.CboInterruptOutByte0.Name = "CboInterruptOutByte0"
			Me.CboInterruptOutByte0.RightToLeft = System.Windows.Forms.RightToLeft.No
			Me.CboInterruptOutByte0.Size = New System.Drawing.Size(72, 22)
			Me.CboInterruptOutByte0.TabIndex = 2
			' 
			' LstResults
			' 
			Me.LstResults.BackColor = System.Drawing.SystemColors.Window
			Me.LstResults.Cursor = System.Windows.Forms.Cursors.Default
			Me.LstResults.Font = New System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.LstResults.ForeColor = System.Drawing.SystemColors.WindowText
			Me.LstResults.HorizontalScrollbar = True
			Me.LstResults.ItemHeight = 16
			Me.LstResults.Location = New System.Drawing.Point(16, 248)
			Me.LstResults.Name = "LstResults"
			Me.LstResults.RightToLeft = System.Windows.Forms.RightToLeft.No
			Me.LstResults.Size = New System.Drawing.Size(543, 356)
			Me.LstResults.TabIndex = 0
			' 
			' CmdFindDevice
			' 
			Me.CmdFindDevice.Font = New System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.CmdFindDevice.Location = New System.Drawing.Point(446, 44)
			Me.CmdFindDevice.Name = "CmdFindDevice"
			Me.CmdFindDevice.Size = New System.Drawing.Size(102, 59)
			Me.CmdFindDevice.TabIndex = 11
			Me.CmdFindDevice.Text = "Find My Device"
			'			Me.CmdFindDevice.Click += New System.EventHandler(Me.cmdFindDevice_Click)
			' 
			' FraControWriteTransfer
			' 
			Me.FraControWriteTransfer.BackColor = System.Drawing.SystemColors.Control
			Me.FraControWriteTransfer.Controls.Add(Me.CboControlWriteByte1)
			Me.FraControWriteTransfer.Controls.Add(Me.CboControlWriteByte0)
			Me.FraControWriteTransfer.Controls.Add(Me.CmdControlWriteTransfer)
			Me.FraControWriteTransfer.Font = New System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.FraControWriteTransfer.ForeColor = System.Drawing.SystemColors.ControlText
			Me.FraControWriteTransfer.Location = New System.Drawing.Point(16, 25)
			Me.FraControWriteTransfer.Name = "FraControWriteTransfer"
			Me.FraControWriteTransfer.RightToLeft = System.Windows.Forms.RightToLeft.No
			Me.FraControWriteTransfer.Size = New System.Drawing.Size(193, 96)
			Me.FraControWriteTransfer.TabIndex = 12
			Me.FraControWriteTransfer.TabStop = False
			Me.FraControWriteTransfer.Text = "Control Write Transfer"
			' 
			' CboControlWriteByte1
			' 
			Me.CboControlWriteByte1.BackColor = System.Drawing.SystemColors.Window
			Me.CboControlWriteByte1.Cursor = System.Windows.Forms.Cursors.Default
			Me.CboControlWriteByte1.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList
			Me.CboControlWriteByte1.Font = New System.Drawing.Font("Arial", 8.0F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.CboControlWriteByte1.ForeColor = System.Drawing.SystemColors.WindowText
			Me.CboControlWriteByte1.Location = New System.Drawing.Point(16, 58)
			Me.CboControlWriteByte1.Name = "CboControlWriteByte1"
			Me.CboControlWriteByte1.RightToLeft = System.Windows.Forms.RightToLeft.No
			Me.CboControlWriteByte1.Size = New System.Drawing.Size(67, 22)
			Me.CboControlWriteByte1.TabIndex = 15
			' 
			' CboControlWriteByte0
			' 
			Me.CboControlWriteByte0.BackColor = System.Drawing.SystemColors.Window
			Me.CboControlWriteByte0.Cursor = System.Windows.Forms.Cursors.Default
			Me.CboControlWriteByte0.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList
			Me.CboControlWriteByte0.Font = New System.Drawing.Font("Arial", 8.0F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.CboControlWriteByte0.ForeColor = System.Drawing.SystemColors.WindowText
			Me.CboControlWriteByte0.Location = New System.Drawing.Point(16, 30)
			Me.CboControlWriteByte0.Name = "CboControlWriteByte0"
			Me.CboControlWriteByte0.RightToLeft = System.Windows.Forms.RightToLeft.No
			Me.CboControlWriteByte0.Size = New System.Drawing.Size(67, 22)
			Me.CboControlWriteByte0.TabIndex = 14
			' 
			' CmdControlWriteTransfer
			' 
			Me.CmdControlWriteTransfer.Font = New System.Drawing.Font("Arial", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.CmdControlWriteTransfer.Location = New System.Drawing.Point(102, 30)
			Me.CmdControlWriteTransfer.Name = "CmdControlWriteTransfer"
			Me.CmdControlWriteTransfer.Size = New System.Drawing.Size(71, 36)
			Me.CmdControlWriteTransfer.TabIndex = 13
			Me.CmdControlWriteTransfer.Text = "Send"
			Me.CmdControlWriteTransfer.UseVisualStyleBackColor = True
			'			Me.CmdControlWriteTransfer.Click += New System.EventHandler(Me.cmdControlWriteTransfer_Click)
			' 
			' FrmMain
			' 
			Me.ClientSize = New System.Drawing.Size(579, 622)
			Me.Controls.Add(Me.FraControWriteTransfer)
			Me.Controls.Add(Me.CmdFindDevice)
			Me.Controls.Add(Me.FraControlReadTransfer)
			Me.Controls.Add(Me.FraBulkTransfer)
			Me.Controls.Add(Me.FraInterruptTransfer)
			Me.Controls.Add(Me.LstResults)
			Me.Font = New System.Drawing.Font("Arial", 8.0F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, (CByte(0)))
			Me.Location = New System.Drawing.Point(21, 28)
			Me.Name = "FrmMain"
			Me.StartPosition = System.Windows.Forms.FormStartPosition.Manual
			Me.Text = "WinUSB Demo"
			'			Me.Closed += New System.EventHandler(Me.frmMain_Closed)
			'			Me.Load += New System.EventHandler(Me.frmMain_Load)
			Me.FraControlReadTransfer.ResumeLayout(False)
			Me.FraBulkTransfer.ResumeLayout(False)
			Me.FraBulkTransfer.PerformLayout()
			Me.FraInterruptTransfer.ResumeLayout(False)
			Me.FraControWriteTransfer.ResumeLayout(False)
			Me.ResumeLayout(False)

		End Sub

#End Region

		' *************************************************************************************
		' Device-specific values. Edit to match your device.

		' If using the sytem-provided winusb.inf, this GUID must match the GUID in the device's 
		' extended properties OS feature descriptor.
		' If using a vendor-provided INF file, this GUID must match the GUID in the INF file.
		' The GUID should be unique to your device.
		' To create a GUID in Visual Studio, click Tools > Create GUID.

		Private Const DeviceInterfaceGuid As String = "{ecceff35-146c-4ff3-acd9-8f992d09acdd}"

		' WMI functions use these values to detect device arrival and removal and get device properties:

		Private Const VendorID As String = "0925"
		Private Const ProductID As String = "150C"

		' *************************************************************************************

		Private _deviceArrivedWatcher As ManagementEventWatcher
		Private _deviceDetected As Boolean
		Private _deviceHandle As SafeFileHandle
		Private _deviceNotificationHandle As IntPtr
		Private _deviceReady As Boolean
		Private _deviceRemovedWatcher As ManagementEventWatcher
		Private _myDeviceInfo As New WinUsbCommunications.DeviceInfo()
		Private ReadOnly _myDeviceManagement As New DeviceManagement()
		Private ReadOnly _myWinUsbCommunications As New WinUsbCommunications()
		Private _winUsbHandle As IntPtr

		Private Enum FormActions
			AddItemToListBox
			EnableCmdSendandReceiveViaBulkTransfers
			EnableCmdSendandReceiveViaInterruptTransfers
			ScrollToBottomOfListBox
		End Enum

		Private Enum WmiDeviceProperties
			Caption
			Description
			Manufacturer
			Name
			CompatibleID ' Returns System.String[]
			PNPDeviceID
			DeviceID
			ClassGUID
			'Availability ' Always returns zero.
		End Enum

		Friend FrmMy As FrmMain

		'''  <summary>
		'''  Define a delegate with the same parameters as AccessForm.
		'''  Used in accessing the application's form from a different thread.
		'''  </summary>

		Private Delegate Sub MarshalToForm(ByVal action As String, ByVal textToAdd As String)

		'''  <summary>
		'''  Define a class of delegates with the same parameters as 
		'''  WinUsbDevice.ReadViaBulkTransfer.
		'''  Used for asynchronous reads from the device.
		'''  </summary>

		Private Delegate Sub ReceiveFromDeviceDelegate(ByVal winUsbHandle As IntPtr, ByVal myDeviceInfo As WinUsbCommunications.DeviceInfo, ByVal bytesToRead As UInt32, ByRef dataBuffer() As Byte, ByRef bytesRead As UInt32, ByRef success As Boolean)

		'''  <summary>
		'''  Define a class of delegates with the same parameters as 
		'''  WinUsbDevice.SendViaBulkTransfer.
		'''  Used for asynchronous writes to the device.
		'''  </summary>		

		Private Delegate Sub SendToDeviceDelegate(ByVal winUsbHandle As IntPtr, ByVal myDevInfo As WinUsbCommunications.DeviceInfo, ByVal bufferLength As UInt32, ByVal buffer() As Byte, ByRef lengthTransferred As UInt32, ByRef success As Boolean)

		'''  <summary>
		'''  Performs various application-specific functions that
		'''  involve accessing the application's form.
		'''  </summary>
		'''  
		'''  <param name="action"> A String that names the action to perform on the form. </param>
		'''  <param name="formText"> Text to display on the form or an empty String. </param>
		'''  
		''' <remarks>
		'''  In asynchronous calls to WinUsb_ReadPipe, the callback function 
		'''  uses this routine to access the application's form, which runs in 
		'''  a different thread.
		''' </remarks>
		''' 
		Private Sub AccessForm(ByVal action As String, ByVal formText As String)
			Try
				'  Select an action to perform on the form:

				Select Case action
					Case "AddItemToListBox"

						LstResults.Items.Add(formText)

					Case "EnableCmdSendandReceiveViaBulkTransfers"

						CmdSendandReceiveViaBulkTransfers.Enabled = True
						CmdSendandReceiveViaBulkTransfers.Focus()

					Case "EnableCmdSendandReceiveViaInterruptTransfers"

						CmdSendAndReceiveViaInterruptTransfers.Enabled = True
						CmdSendAndReceiveViaInterruptTransfers.Focus()

					Case "ScrollToBottomOfListBox"

						LstResults.SelectedIndex = LstResults.Items.Count - 1
				End Select
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Add a handler to detect arrival of devices.
		'''  </summary>

		Private Sub AddDeviceArrivedHandler()
			Const pollingIntervalSeconds As Int32 = 3
			Dim scope = New ManagementScope("root\CIMV2")
			scope.Options.EnablePrivileges = True

			Try
				Dim q = New WqlEventQuery()
				q.EventClassName = "__InstanceCreationEvent"
				q.WithinInterval = New TimeSpan(0, 0, pollingIntervalSeconds)
				q.Condition = "TargetInstance ISA 'Win32_USBControllerdevice'"
				_deviceArrivedWatcher = New ManagementEventWatcher(scope, q)
				AddHandler _deviceArrivedWatcher.EventArrived, AddressOf DeviceAdded

				_deviceArrivedWatcher.Start()
			Catch e As Exception
				Console.WriteLine(e.Message)
				If _deviceArrivedWatcher IsNot Nothing Then
					_deviceArrivedWatcher.Stop()
				End If
			End Try
		End Sub

		'''  <summary>
		'''  Add a handler to detect removal of devices.
		'''  </summary>

		Private Sub AddDeviceRemovedHandler()
			Const pollingIntervalSeconds As Int32 = 3
			Dim scope = New ManagementScope("root\CIMV2")
			scope.Options.EnablePrivileges = True

			Try
				Dim q = New WqlEventQuery()
				q.EventClassName = "__InstanceDeletionEvent"
				q.WithinInterval = New TimeSpan(0, 0, pollingIntervalSeconds)
				q.Condition = "TargetInstance ISA 'Win32_USBControllerdevice'"
				_deviceRemovedWatcher = New ManagementEventWatcher(scope, q)
				AddHandler _deviceRemovedWatcher.EventArrived, AddressOf DeviceRemoved
				_deviceRemovedWatcher.Start()
			Catch e As Exception
				Console.WriteLine(e.Message)
				If _deviceRemovedWatcher IsNot Nothing Then
					_deviceRemovedWatcher.Stop()
				End If
			End Try
		End Sub

		'''  <summary>
		'''  Calls a routine to initiate a Control Read transfer. (Data stage is device to host.)
		'''  </summary>

		Private Sub cmdControlReadTransfer_Click(ByVal sender As Object, ByVal e As EventArgs) Handles CmdControlReadTransfer.Click
			RequestControlReadTransfer()
		End Sub

		'''  <summary>
		'''  Calls a routine to initiate a Control Write transfer. (Data stage is host to device.)
		'''  </summary>
		'''   
		Private Sub cmdControlWriteTransfer_Click(ByVal sender As Object, ByVal e As EventArgs) Handles CmdControlWriteTransfer.Click
			RequestControlWriteTransfer()
		End Sub

		'''  <summary>
		'''  Search for a specific device.
		'''  </summary>

		Private Sub cmdFindDevice_Click(ByVal sender As Object, ByVal e As EventArgs) Handles CmdFindDevice.Click
			FindMyDevice()
		End Sub

		'''  <summary>
		'''  Calls a routine to exchange data via bulk transfers.
		'''  </summary>

		Private Sub cmdSendAndReceiveViaBulkTransfers_Click(ByVal eventSender As Object, ByVal eventArgs As EventArgs) Handles CmdSendandReceiveViaBulkTransfers.Click
			' Don't allow another transfer request until this one completes.

			CmdSendandReceiveViaBulkTransfers.Enabled = False

			SendAndReceiveViaBulkTransfers()
		End Sub

		'''  <summary>
		'''  Calls a routine to exchange data via interrupt transfers.
		'''  </summary>
		'''  
		Private Sub cmdSendAndReceiveViaInterruptTransfers_Click(ByVal sender As Object, ByVal e As EventArgs) Handles CmdSendAndReceiveViaInterruptTransfers.Click
			' Don't allow another transfer request until this one completes.

			CmdSendAndReceiveViaInterruptTransfers.Enabled = False

			SendAndReceiveViaInterruptTransfers()
		End Sub

		'''  <summary>
		'''  Called on arrival of any device.
		'''  Calls a routine that searches to see if the desired device is present.
		'''  </summary>

		Private Sub DeviceAdded(ByVal sender As Object, ByVal e As EventArrivedEventArgs)
			Try
				Console.WriteLine("A USB device has been inserted")

				_deviceDetected = FindDeviceUsingWmi()
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Add handlers to detect device arrival and removal.
		'''  </summary>

		Private Sub DeviceNotificationsStart()
			AddDeviceArrivedHandler()
			AddDeviceRemovedHandler()
		End Sub

		'''  <summary>
		'''  Stop receiving notifications about device arrival and removal
		'''  </summary>

		Private Sub DeviceNotificationsStop()
			Try
				If _deviceArrivedWatcher IsNot Nothing Then
					_deviceArrivedWatcher.Stop()
				End If
				If _deviceRemovedWatcher IsNot Nothing Then
					_deviceRemovedWatcher.Stop()
				End If
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Called on removal of any device.
		'''  Calls a routine that searches to see if the desired device is still present.
		'''  </summary>
		''' 
		Private Sub DeviceRemoved(ByVal sender As Object, ByVal e As EventArgs)
			Try
				Console.WriteLine("A USB device has been removed")

				_deviceDetected = FindDeviceUsingWmi()
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try

		End Sub

		'''  <summary>
		'''  Get and display the device's speed in the list box.
		'''  </summary>		

		Private Sub DisplayDeviceSpeed()
			Try
				Dim speed = ""

				WinUsbCommunications.QueryDeviceSpeed(_winUsbHandle, _myDeviceInfo)

				Select Case _myDeviceInfo.DeviceSpeed
					Case 1
						speed = "low or full speed"
					Case 3
						speed = "high speed or SuperSpeed"
				End Select

				LstResults.Items.Add("Device is " & speed)
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Provides a central mechanism for exception handling.
		'''  Displays a message box that describes the exception.
		'''  </summary>
		'''  
		'''  <param name="Name"> the module where the exception occurred. </param>
		'''  <param name="e"> the exception </param>

		Friend Shared Sub DisplayException(ByVal Name As String, ByVal e As Exception)
			Try
				'  Create an error message.

				Dim message As String = "Exception: " & e.Message & Environment.NewLine & "Module: " & Name & Environment.NewLine & "Method: " & e.TargetSite.Name

				Const caption As String = "Unexpected Exception"

				MessageBox.Show(message, caption, MessageBoxButtons.OK)
				Debug.Write(message)
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Use WMI to find a device by Vendor ID and Product ID. If found, display device properties.
		'''  Can also search by ClassGUID, description, and other properties. 
		'''  The WMI functions don't detect the device interface GUID so you need to use another property or properties.
		'''  </summary>
		''' 
		Private Function FindDeviceUsingWmi() As Boolean
			Try
				' ClassGUID value from Windows-provided winusb.inf for use if you want to search by ClassGUID.

				Const classGuid As String = "88bae032-5a81-49f0-bc3d-a4ff138216d6"
				Const deviceIdString As String = "USB\VID_" & VendorID & "&PID_" & ProductID

				_deviceDetected = False
				Dim searcher = New ManagementObjectSearcher("root\CIMV2", "SELECT * FROM Win32_PnPEntity")

				For Each queryObj As ManagementObject In searcher.Get()
					' More query exampless for the device search below:

					' Search for any WinUSB device:
					' if (queryObj["ClassGUID"].ToString().Contains(classGuid))

					' Search by description:
					' if (queryObj["Description"].ToString().ToLower().Contains("winusb"))

					' Prepend "@" in string below to treat backslash as a normal character (not escape character):

					'const String deviceIdString = @"USB\VID_" + VendorID + "&PID_" + ProductID;

					If queryObj("PNPDeviceID").ToString().Contains(deviceIdString) Then
						_deviceDetected = True
						MyMarshalToForm(FormActions.AddItemToListBox.ToString(), "--------")
						MyMarshalToForm(FormActions.AddItemToListBox.ToString(), "My device found (WMI):")

						For Each wmiDeviceProperty As WmiDeviceProperties In System.Enum.GetValues(GetType(WmiDeviceProperties))
							'MyMarshalToForm(FormActions.AddItemToListBox.ToString(), CType((wmiDeviceProperty.ToString() & ": " & queryObj(wmiDeviceProperty.ToString()).ToString), String))
							MyMarshalToForm(FormActions.AddItemToListBox.ToString(), wmiDeviceProperty.ToString() & ": " & queryObj(wmiDeviceProperty.ToString()).ToString)
							Console.WriteLine(wmiDeviceProperty.ToString() & ": {0}", queryObj(wmiDeviceProperty.ToString()))
						Next wmiDeviceProperty
						MyMarshalToForm(FormActions.AddItemToListBox.ToString(), "--------")
						MyMarshalToForm(FormActions.ScrollToBottomOfListBox.ToString(), "")
					End If
				Next queryObj
				If Not _deviceDetected Then
					_deviceReady = False
					MyMarshalToForm(FormActions.AddItemToListBox.ToString(), "My device not found (WMI)")
					MyMarshalToForm(FormActions.ScrollToBottomOfListBox.ToString(), "")
				End If
				Return _deviceDetected
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Function

		'''  <summary>
		'''  If a device with the specified device interface GUID hasn't been previously detected,
		'''  look for it. If found, open a handle to the device.
		'''  </summary>
		'''  
		'''  <returns>
		'''  True if the device is detected, False if not detected.
		'''  </returns>

		Private Sub FindMyDevice()
			Try
				Dim devicePathName = ""
				Dim lastDevice As Boolean
				Const pipeTimeout As UInteger = 2000

				If Not (_deviceReady) Then
					' Convert the device interface GUID String to a GUID object: 

					Dim winUsbDemoGuid = New Guid(DeviceInterfaceGuid)

					' Fill an array with the device path names of all attached devices with matching GUIDs.

					Dim deviceFound = _myDeviceManagement.FindDeviceFromGuid(winUsbDemoGuid, devicePathName)

					If deviceFound Then
						_deviceHandle = _myWinUsbCommunications.GetDeviceHandle(devicePathName)

						If Not _deviceHandle.IsInvalid Then
							_deviceReady = True

							_myWinUsbCommunications.InitializeDevice(_deviceHandle, _winUsbHandle, _myDeviceInfo, pipeTimeout)

							DisplayDeviceSpeed()
						Else
							' There was a problem in retrieving the information.

							_deviceReady = False
							_myWinUsbCommunications.CloseDeviceHandle(_deviceHandle, _winUsbHandle)
							LstResults.Items.Add("Device not found.")
						End If
					Else
						CmdSendandReceiveViaBulkTransfers.Enabled = True
						CmdSendAndReceiveViaInterruptTransfers.Enabled = True
					End If
				Else
					LstResults.Items.Add("The device has been detected.")
					DisplayDeviceSpeed()
				End If

				' Display device information.

				FindDeviceUsingWmi()
				ScrollToBottomOfListBox()
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Perform shutdown operations.
		'''  </summary>

		Private Sub frmMain_Closed(ByVal eventSender As Object, ByVal eventArgs As EventArgs) Handles MyBase.Closed
			Shutdown()
		End Sub

		'''  <summary>
		'''  Perform startup operations.
		'''  </summary>

		Private Sub frmMain_Load(ByVal eventSender As Object, ByVal eventArgs As EventArgs) Handles MyBase.Load
			FrmMy = Me
			Startup()
		End Sub

		'''  <summary>
		'''  Retrieves received data from a bulk endpoint.
		'''  This routine is called automatically when myWinUsbDevice.ReadViaBulkTransfer
		'''  returns. The routine calls several marshaling routines to access the main form.       
		'''  </summary>
		'''  
		'''  <param name="ar"> An object containing status information about the 
		'''  asynchronous operation.</param>
		'''  
		Private Sub GetBulkDataSent(ByVal ar As IAsyncResult)
			Try
				Dim bytesSent As UInt32 = 0
				Dim myEncoder = New ASCIIEncoding()
				Dim sentDataBuffer() As Byte
				Dim receivedtext As String = ""
				Dim success As Boolean = False

				' Define a delegate using the IAsyncResult object.

				Dim deleg = (DirectCast(ar.AsyncState, SendToDeviceDelegate))

				' Get the IAsyncResult object and the values of other paramaters that the
				' BeginInvoke method passed by reference.

				deleg.EndInvoke(bytesSent, success, ar)

				' Display the received data in the form's list box.

				If (ar.IsCompleted AndAlso success) Then
					MyMarshalToForm(FormActions.AddItemToListBox.ToString(), "Data sent via bulk transfer:")
				Else
					MyMarshalToForm(FormActions.AddItemToListBox.ToString(), "The attempt to send bulk data has failed.")
				End If

				MyMarshalToForm(FormActions.ScrollToBottomOfListBox.ToString(), "")

				' Enable requesting another transfer.

				MyMarshalToForm(FormActions.EnableCmdSendandReceiveViaBulkTransfers.ToString(), "")
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Retrieves received data from a bulk endpoint.
		'''  This routine is called automatically when myWinUsbDevice.ReceiveViaBulkTransfer
		'''  returns. The routine calls several marshaling routines to access the main form.       
		'''  </summary>
		'''  
		'''  <param name="ar"> An object containing status information about the 
		'''  asynchronous operation.</param>
		'''  
		Private Sub GetBulkDataReceived(ByVal ar As IAsyncResult)
			Try
				Dim bytesRead As UInt32 = 0
				Dim myEncoder = New ASCIIEncoding()
				Dim success = False

				Dim receivedDataBuffer() As Byte = Nothing

				' Define a delegate using the IAsyncResult object.

				Dim deleg = (DirectCast(ar.AsyncState, ReceiveFromDeviceDelegate))

				' Get the IAsyncResult object and the values of other paramaters that the
				' BeginInvoke method passed ByRef.

				deleg.EndInvoke(receivedDataBuffer, bytesRead, success, ar)

				' Display the received data in the form's list box.

				If (ar.IsCompleted AndAlso success) Then
					MyMarshalToForm(FormActions.AddItemToListBox.ToString(), "Data received via bulk transfer:")

					'  Convert the received bytes to a String for display.

					Dim receivedtext As String = myEncoder.GetString(receivedDataBuffer)

					MyMarshalToForm(FormActions.AddItemToListBox.ToString(), receivedtext)
				Else
					MyMarshalToForm(FormActions.AddItemToListBox.ToString(), "The attempt to read bulk data has failed.")
				End If

				MyMarshalToForm(FormActions.ScrollToBottomOfListBox.ToString(), "")

				' Enable requesting another transfer.

				MyMarshalToForm(FormActions.EnableCmdSendandReceiveViaBulkTransfers.ToString(), "")
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Initializes elements on the form.
		'''  </summary>

		Private Sub InitializeDisplay()
			Try
				' Create dropdown list boxes.

				For i = 0 To 255
					Dim byteValue As String = String.Format("{0:X2} ", i)

					FrmMy.CboInterruptOutByte0.Items.Insert(i, byteValue)
					FrmMy.CboInterruptOutByte1.Items.Insert(i, byteValue)
					FrmMy.CboControlWriteByte0.Items.Insert(i, byteValue)
					FrmMy.CboControlWriteByte1.Items.Insert(i, byteValue)
				Next i

				' Select a default value for each box

				FrmMy.CboInterruptOutByte0.SelectedIndex = 0
				FrmMy.CboInterruptOutByte1.SelectedIndex = 128
				FrmMy.CboControlWriteByte0.SelectedIndex = 65
				FrmMy.CboControlWriteByte1.SelectedIndex = 53

				If Not (_myWinUsbCommunications.IsWindowsXpOrLater()) Then
					LstResults.Items.Add("The operating system is not Windows XP or later.")
					LstResults.Items.Add("The WinUsb driver requires Windows XP or later.")
				End If
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Enables accessing a form from another thread 
		'''  </summary>
		'''  
		'''  <param name="action"> A String that names the action to perform on the form. </param>
		'''  <param name="textToDisplay"> Text that the form displays or uses for 
		'''  another purpose. Actions that don't use text ignore this parameter. </param>

		Private Sub MyMarshalToForm(ByVal action As String, ByVal textToDisplay As String)
			Try
				Dim args() As Object = {action, textToDisplay}

				'  The AccessForm routine contains the code that accesses the form.

				Dim marshalToFormDelegate As MarshalToForm = AddressOf AccessForm

				'  Execute AccessForm, passing the parameters in args.

				Invoke(marshalToFormDelegate, args)
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Initiates a Control Read transfer. (Data stage is device to host.)
		'''  </summary>
		'''  
		Private Sub RequestControlReadTransfer()
			Try
				Dim dataStage = New Byte(1) {}

				' If the device hasn't been detected or was removed, look for the device.

				If Not (_deviceReady) Then
					FindMyDevice()
				End If

				If _deviceReady Then
					Dim success = _myWinUsbCommunications.DoControlReadTransfer(_winUsbHandle, dataStage)

					Dim formText As String
					If success Then
						formText = "Bytes received via Control Read transfer:"
						AccessForm(FormActions.AddItemToListBox.ToString(), formText)

						For i As Int32 = 0 To dataStage.Length - 1
							formText = String.Format("{0:X2} ", dataStage(i))
							AccessForm(FormActions.AddItemToListBox.ToString(), formText)
						Next i
					Else
						formText = "Control Read transfer failed."
						AccessForm(FormActions.AddItemToListBox.ToString(), formText)
					End If
				End If
				ScrollToBottomOfListBox()
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Initiates a Control Write transfer. (Data stage is host to device.)
		'''  Converts hex strings from combo boxes to byte values for sending.
		'''  </summary>
		'''  
		Private Sub RequestControlWriteTransfer()
			Try
				Dim dataStage = New Byte(1) {}

				' If the device hasn't been detected, was removed, or timed out on a previous attempt
				' to access it, look for the device.

				If Not (_deviceReady) Then
					FindMyDevice()
				End If

				If _deviceReady Then
					' Get the value to send as a hex String.

					Dim comboBoxText = Convert.ToString(CboControlWriteByte0.SelectedItem).TrimEnd(Nothing)

					' Convert the String to a byte.

					dataStage(0) = Convert.ToByte(String.Format("{0:X2}", comboBoxText), 16)

					' Get the value to send as a hex String.

					comboBoxText = Convert.ToString(CboControlWriteByte1.SelectedItem).TrimEnd(Nothing)

					' Convert the String to a byte.

					dataStage(1) = Convert.ToByte(String.Format("{0:X2}", comboBoxText), 16)

					Dim success = _myWinUsbCommunications.DoControlWriteTransfer(_winUsbHandle, dataStage)

					Dim formText As String
					If success Then
						formText = "Control Write transfer completed successfully."
					Else
						formText = "Control Write transfer failed."
					End If

					AccessForm(FormActions.AddItemToListBox.ToString(), formText)
				End If
				ScrollToBottomOfListBox()
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Initiates a read operation from a bulk IN endpoint.
		'''  To enable reading without blocking the main thread, uses an asynchronous delegate.
		'''  </summary>
		'''  
		'''  <param name="bytesToRead"> The number of bytes to read </param>		
		'''  <param name="dataBuffer"> Buffer to hold the bytes read </param>
		'''  <param name="bytesRead"> The number of bytes read </param>
		'''  <param name="success"> True on success </param>		

		Private Sub RequestToReceiveDataViaBulkTransfer(ByVal bytesToRead As UInt32, ByVal dataBuffer() As Byte, ByRef bytesRead As UInt32, ByRef success As Boolean)
			Try
				If dataBuffer Is Nothing Then
					Throw New ArgumentNullException("dataBuffer")
				End If
				Dim ar As IAsyncResult = Nothing

				'  Define a delegate for the ReadViaBulkTransfer method of WinUsbDevice.

				Dim myReceiveFromDeviceDelegate As New ReceiveFromDeviceDelegate(AddressOf _myWinUsbCommunications.ReceiveDataViaBulkTransfer)
				'Dim myReceiveFromDeviceDelegate As ReceiveFromDeviceDelegate = _myWinUsbCommunications.ReceiveDataViaBulkTransfer

				'  The BeginInvoke method calls MyWinUsbDevice.ReceiveViaBulkTransfer to attempt 
				'  to read data. The method has the same parameters as ReceiveViaBulkTransfer,
				'  plus two additional parameters:
				'  GetBulkDataReceived is the callback routine that executes when 
				'  ReceiveViaBulkTransfer returns.
				'  MyReceiveFromDeviceDelegate is the asynchronous delegate object.

				myReceiveFromDeviceDelegate.BeginInvoke(_winUsbHandle, _myDeviceInfo, bytesToRead, dataBuffer, bytesRead, success, AddressOf GetBulkDataReceived, myReceiveFromDeviceDelegate)
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Initiates a read operation from an interrupt IN endpoint.
		'''  Synchronous, blocks until data received or timeout.
		'''  </summary>
		'''  
		'''  <param name="bytesToRead"> The number of bytes to read </param>		
		'''  <param name="dataBuffer"> Buffer to hold the bytes read </param>
		'''  <param name="bytesRead"> The number of bytes read </param>
		'''  <param name="success"> True on success </param>  		

		Private Sub RequestToReceiveDataViaInterruptTransfer(ByVal bytesToRead As UInt32, ByVal dataBuffer() As Byte, ByRef bytesRead As UInt32, ByRef success As Boolean)
			Try
				If dataBuffer Is Nothing Then
					Throw New ArgumentNullException("dataBuffer")
				End If
				success = _myWinUsbCommunications.ReceiveDataViaInterruptTransfer(_winUsbHandle, _myDeviceInfo, bytesToRead, dataBuffer, bytesRead)

				If success Then
					MyMarshalToForm(FormActions.AddItemToListBox.ToString(), "Data received via interrupt transfer:")

					MyMarshalToForm(FormActions.AddItemToListBox.ToString(), " Received Data:")

					For i As Int32 = 0 To dataBuffer.GetUpperBound(0)
						' Convert the byte value to a 2-character hex String.

						Dim byteValue = String.Format("{0:X2} ", dataBuffer(i))

						MyMarshalToForm(FormActions.AddItemToListBox.ToString(), " " & byteValue)
					Next i
				Else
					MyMarshalToForm(FormActions.AddItemToListBox.ToString(), "The attempt to read interrupt data has failed.")
				End If

				MyMarshalToForm(FormActions.ScrollToBottomOfListBox.ToString(), "")

				' Enable requesting another transfer.

				MyMarshalToForm(FormActions.EnableCmdSendandReceiveViaInterruptTransfers.ToString(), "")
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Initiates a write operation to a bulk OUT endpoint.
		'''  To enable writing without blocking the main thread, uses an asynchronous delegate.
		'''  </summary>
		'''  
		'''  <param name="bytesToSend"> The number of bytes to send </param> 
		'''  <param name="dataBuffer"> Buffer to hold the bytes to send </param> 
		'''  <param name="bytesSent"> The number of bytes send </param>
		'''  <param name="success"> True on success </param>		

		Private Sub RequestToSendDataViaBulkTransfer(ByVal bytesToSend As UInt32, ByVal dataBuffer() As Byte, ByRef bytesSent As UInt32, ByRef success As Boolean)
			Try
				If dataBuffer Is Nothing Then
					Throw New ArgumentNullException("dataBuffer")
				End If
				Dim ar As IAsyncResult = Nothing

				'  Define a delegate for the ReadViaBulkTransfer method of WinUsbDevice.
				Dim mySendToDeviceDelegate As _
				New SendToDeviceDelegate(AddressOf _myWinUsbCommunications.SendDataViaBulkTransfer)


				'Dim mySendToDeviceDelegate As SendToDeviceDelegate = _myWinUsbCommunications.SendDataViaBulkTransfer

				'  The BeginInvoke method calls MyWinUsbDevice.SendViaBulkTransfer to attempt 
				'  to read data. The method has the same parameters as SendViaBulkTransfer,
				'  plus two additional parameters:
				'  GetBulkDataSent is the callback routine that executes when 
				'  SendViaBulkTransfer returns.
				'  MySendToDeviceDelegate is the asynchronous delegate object.

				mySendToDeviceDelegate.BeginInvoke(_winUsbHandle, _myDeviceInfo, bytesToSend, dataBuffer, bytesSent, success, AddressOf GetBulkDataSent, mySendToDeviceDelegate)
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Initiates a write operation to an interrupt OUT endpoint.
		'''  Synchronous, blocks until data sent or timeout.
		'''  </summary>
		'''  
		'''  <param name="bytesToSend"> The number of bytes to send </param> 
		'''  <param name="dataBuffer"> Buffer to hold the bytes to send </param> 
		'''  <param name="bytesSent"> The number of bytes send </param>
		'''  <param name="success"> True on success </param>	

		Private Sub RequestToSendDataViaInterruptTransfer(ByVal bytesToSend As UInt32, ByVal dataBuffer() As Byte, ByRef bytesSent As UInt32, ByRef success As Boolean)
			Try
				If dataBuffer Is Nothing Then
					Throw New ArgumentNullException("dataBuffer")
				End If
				success = _myWinUsbCommunications.SendDataViaInterruptTransfer(_winUsbHandle, _myDeviceInfo, bytesToSend, dataBuffer, bytesSent)

				If success Then
					MyMarshalToForm(FormActions.AddItemToListBox.ToString(), "Data sent via interrupt transfer:")
				Else
					MyMarshalToForm(FormActions.AddItemToListBox.ToString(), "The attempt to send interrupt data has failed.")
				End If

				MyMarshalToForm(FormActions.ScrollToBottomOfListBox.ToString(), "")

				' Enable requesting another transfer.

				MyMarshalToForm(FormActions.EnableCmdSendandReceiveViaInterruptTransfers.ToString(), "")
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Scroll to the bottom of the list box and trim if needed.
		'''  </summary>

		Private Sub ScrollToBottomOfListBox()
			Try
				LstResults.SelectedIndex = LstResults.Items.Count - 1

				' If the list box is getting too large, trim its contents.

				If LstResults.Items.Count > 1000 Then
					For i As Int32 = 1 To 800
						LstResults.Items.RemoveAt(i)
					Next i
				End If
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Initiates sending data via a bulk transfer, then receiving data via a bulk transfer.
		'''  </summary>

		Private Sub SendAndReceiveViaBulkTransfers()
			Try
				Dim bytesWritten As UInt32 = 0
				Dim myEncoder = New ASCIIEncoding()
				Dim success = False

				'  Get data to send from the textbox.

				Dim dataToSend = TxtBulkDataToSend.Text

				If dataToSend.Length > 64 Then
					dataToSend = dataToSend.Substring(0, 64)
				End If

				'  Convert the String to a byte array.

				Dim databuffer = myEncoder.GetBytes(dataToSend)

				Dim bytesToSend = Convert.ToUInt32(databuffer.Length)

				' If the device hasn't been detected, look for it.

				If Not (_deviceReady) Then
					FindMyDevice()
				End If

				'INSTANT VB NOTE: The variable dataBuffer was renamed since Visual Basic will not allow local variables with the same name as parameters or other local variables:
				Dim dataBuffer_Renamed = New Byte(63) {}
				If _deviceReady Then
					RequestToSendDataViaBulkTransfer(bytesToSend, databuffer, bytesWritten, success)

					Dim bytesRead As UInt32 = 0
					Dim bytesToRead As UInt32 = Convert.ToUInt32(64)

					RequestToReceiveDataViaBulkTransfer(bytesToRead, dataBuffer_Renamed, bytesRead, success)
				End If
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Initiates sending data via an interrupt transfer, then retrieving data 
		'''  via an interrupt transfer.
		'''  Converts hex strings from combo boxes to byte values for sending.
		'''  </summary>

		Private Sub SendAndReceiveViaInterruptTransfers()
			Try
				Dim bytesWritten As UInt32 = 0
				Dim dataBuffer = New Byte(1) {}
				Dim dataToSend = ""
				Dim formText = ""
				Dim success As Boolean = False

				'  Get bytes to send from combo boxes.

				' Get the value to send as a hex String.

				Dim comboBoxText = Convert.ToString(CboInterruptOutByte0.SelectedItem).TrimEnd(Nothing)

				' Convert the String to a Byte.

				dataBuffer(0) = Convert.ToByte(String.Format("{0:X2}", comboBoxText), 16)

				' Get the value to send as a hex String.

				comboBoxText = Convert.ToString(CboInterruptOutByte1.SelectedItem).TrimEnd(Nothing)

				' Convert the String to a Byte.

				dataBuffer(1) = Convert.ToByte(String.Format("{0:X2}", comboBoxText), 16)

				Dim bytesToSend = Convert.ToUInt32(dataBuffer.Length)

				Dim dataInBuffer = New Byte(1) {}
				Dim bytesRead As UInt32 = 0
				Dim bytesToRead = Convert.ToUInt32(2)


				' If the device hasn't been detected, look for it.

				If Not (_deviceReady) Then
					FindMyDevice()
				End If

				If _deviceReady Then
					RequestToSendDataViaInterruptTransfer(bytesToSend, dataBuffer, bytesWritten, success)
					RequestToReceiveDataViaInterruptTransfer(bytesToRead, dataInBuffer, bytesRead, success)
				End If
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Perform actions that must execute when the program ends.
		'''  </summary>

		Private Sub Shutdown()
			Try
				_myWinUsbCommunications.CloseDeviceHandle(_deviceHandle, _winUsbHandle)


				DeviceNotificationsStop()
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub

		'''  <summary>
		'''  Perform actions that must execute when the program starts.
		'''  </summary>

		Private Sub Startup()
			Try
				Dim myWinUsbCommunications = New WinUsbCommunications()
				InitializeDisplay()

				DeviceNotificationsStart()
				FindMyDevice()
			Catch ex As Exception
				DisplayException(Name, ex)
				Throw
			End Try
		End Sub
	End Class
End Namespace
